#include "main_hal.h"

FDCAN_TxHeaderTypeDef FDCAN_TxHeader; // CAN header, used for transmission
uint8_t FDCAN_TxData[64]; // CAN data to send
FDCAN_RxHeaderTypeDef FDCAN_RxHeader; // CAN header, used for reception
uint8_t FDCAN_RxData[64]; // CAN data, after the reception

/**
 * @brief  start additional Init
 * @param  none
 * @retval none
 */
void Additional_Init ( void )
{
  Additional_TIM6_Init(); // SW Tim
  Additional_CAN_Init();
}

/**
 * @brief  start additional Init Softwaretimer
 * @param  none
 * @retval none
 */
void Additional_TIM6_Init ( void )
{
  HAL_TIM_Base_Start_IT(&htim6);
}

/**
 * @brief  start additional Init for CAN_CANFD
 * @param  none
 * @retval none
 */
void Additional_CAN_Init ( void )
{
  // Change CAN to Silent mode
  HAL_GPIO_WritePin(CAN1_ON_GPIO_Port, CAN1_ON_Pin, GPIO_PIN_SET);
  HAL_GPIO_WritePin(CAN2_ON_GPIO_Port, CAN2_ON_Pin, GPIO_PIN_SET);

  /* Configure and enable Tx Delay Compensation, required for BRS mode.
     TdcOffset default recommended value: DataTimeSeg1 * DataPrescaler
     TdcFilter default recommended value: 0 */
  if ( HAL_FDCAN_ConfigTxDelayCompensation(&hfdcan1, hfdcan1.Init.DataPrescaler * hfdcan1.Init.DataTimeSeg1, 0) != HAL_OK )
  {
    Error_Handler();
  }

  if ( HAL_FDCAN_EnableTxDelayCompensation(&hfdcan1) != HAL_OK )
  {
    Error_Handler();
  }
     /* Start the FDCAN module */
  if ( HAL_FDCAN_Start(&hfdcan1) != HAL_OK )
  {
    Error_Handler();
  }

  if ( HAL_FDCAN_ActivateNotification(&hfdcan1, FDCAN_IT_RX_FIFO0_NEW_MESSAGE, 0) != HAL_OK )
  {
    Error_Handler();
  }

  /* Configure and enable Tx Delay Compensation, required for BRS mode.
     TdcOffset default recommended value: DataTimeSeg1 * DataPrescaler
     TdcFilter default recommended value: 0 */
  if ( HAL_FDCAN_ConfigTxDelayCompensation(&hfdcan2, hfdcan2.Init.DataPrescaler * hfdcan2.Init.DataTimeSeg1, 0) != HAL_OK )
  {
    Error_Handler();
  }

  if ( HAL_FDCAN_EnableTxDelayCompensation(&hfdcan2) != HAL_OK )
  {
    Error_Handler();
  }
 /* Start the FDCAN module */
 if (HAL_FDCAN_Start(&hfdcan2) != HAL_OK)
 {
   Error_Handler();
 }

 if (HAL_FDCAN_ActivateNotification(&hfdcan2, FDCAN_IT_RX_FIFO0_NEW_MESSAGE, 0) != HAL_OK)
 {
   Error_Handler();
 }

  // Change CAN from Silent to Active mode
  HAL_GPIO_WritePin(CAN1_ON_GPIO_Port, CAN1_ON_Pin, GPIO_PIN_RESET);
  HAL_GPIO_WritePin(CAN2_ON_GPIO_Port, CAN2_ON_Pin, GPIO_PIN_RESET);
}

/**
 * @brief  Rx FIFO 0 callback.
 * @param  hfdcan: pointer to an FDCAN_HandleTypeDef structure that contains
 *         the configuration information for the specified FDCAN.
 * @param  RxFifo0ITs: indicates which Rx FIFO 0 interrupts are signalled.
 *         This parameter can be any combination of @arg FDCAN_Rx_Fifo0_Interrupts.
 * @retval None
 */
void HAL_FDCAN_RxFifo0Callback ( FDCAN_HandleTypeDef *hfdcan, uint32_t RxFifo0ITs )
{
  if ( (RxFifo0ITs & FDCAN_IT_RX_FIFO0_NEW_MESSAGE) != RESET )
  {
    if ( hfdcan->Instance == FDCAN1 )
    {
      static uint8_t LEDRed = 0;
      /*
       * LED will toggle every time a message is received
       */
      if ( LEDRed == 0 )
        WriteLED_Red(GPIO_PIN_SET);
      else
        WriteLED_Red(GPIO_PIN_RESET);
      LEDRed = !LEDRed;

      /* Retrieve Rx messages from RX FIFO0 */
      if ( HAL_FDCAN_GetRxMessage(hfdcan, FDCAN_RX_FIFO0, &FDCAN_RxHeader, FDCAN_RxData) == HAL_OK )
      {
        switch ( FDCAN_RxHeader.Identifier )
        {
          case 0x100:
            // Add code here when CAN-ID 0x100 was recieved
            break;
        }
      }
      else
      {
        Error_Handler();
      }
    }
    
    if ( hfdcan->Instance == FDCAN2 )
    {
      static uint8_t LEDGreen = 0;
      /*
       * LED will toggle every time a message is received
       */
      if ( LEDGreen == 0 )
        WriteLED_Green(GPIO_PIN_SET);
      else
        WriteLED_Green(GPIO_PIN_RESET);
      LEDGreen = !LEDGreen;

      /* Retrieve Rx messages from RX FIFO0 */
      if ( HAL_FDCAN_GetRxMessage(hfdcan, FDCAN_RX_FIFO0, &FDCAN_RxHeader, FDCAN_RxData) == HAL_OK )
      {
        switch ( FDCAN_RxHeader.Identifier )
        {
          case 0x102:
            // Add code here when CAN-ID 0x100 was recieved
            break;
        }
      }
      else
      {
        Error_Handler();
      }
    }
  }
}

/**
 * @brief  Send a CAN-message on CAN1-bus
 *         Is the CANID smaller than 0x800 the standard ID frame format is used, otherwise the extended ID frame format is used.
 * @param  CANID: specifies the CANID with which the message is to be sent
 *         DLC: specifies the number of bytes that will be sent
 *         Byte0: Data in Byte 0
 *         Byte1: Data in Byte 1
 *         Byte2: Data in Byte 2
 *         Byte3: Data in Byte 3
 *         Byte4: Data in Byte 4
 *         Byte5: Data in Byte 5
 *         Byte6: Data in Byte 6
 *         Byte7: Data in Byte 7
 * @retval none
 */
void SendCAN1_ClassicMessage ( uint32_t CANID, uint32_t DLC, uint8_t Byte0, uint8_t Byte1, uint8_t Byte2, uint8_t Byte3, uint8_t Byte4, uint8_t Byte5,
    uint8_t Byte6, uint8_t Byte7 )
{
  /* Prepare Tx Header */
  FDCAN_TxHeader.FDFormat = FDCAN_CLASSIC_CAN;
  FDCAN_TxHeader.TxFrameType = FDCAN_DATA_FRAME;
  FDCAN_TxHeader.ErrorStateIndicator = FDCAN_ESI_ACTIVE;
  FDCAN_TxHeader.BitRateSwitch = FDCAN_BRS_OFF;
  FDCAN_TxHeader.TxEventFifoControl = FDCAN_NO_TX_EVENTS;
  FDCAN_TxHeader.MessageMarker = 0;
  FDCAN_TxHeader.Identifier = CANID;
  FDCAN_TxHeader.IdType = FDCAN_STANDARD_ID;
  if ( CANID >= 0x800 )
    FDCAN_TxHeader.IdType = FDCAN_EXTENDED_ID;
  FDCAN_TxHeader.DataLength = DLC;

  /* Fill Data */
  FDCAN_TxData[0] = Byte0;
  FDCAN_TxData[1] = Byte1;
  FDCAN_TxData[2] = Byte2;
  FDCAN_TxData[3] = Byte3;
  FDCAN_TxData[4] = Byte4;
  FDCAN_TxData[5] = Byte5;
  FDCAN_TxData[6] = Byte6;
  FDCAN_TxData[7] = Byte7;

  /* Start the Transmission process */
  if ( HAL_FDCAN_AddMessageToTxFifoQ(&hfdcan1, &FDCAN_TxHeader, FDCAN_TxData) != HAL_OK )
  {
    /* Transmission request Error */
    // Error_Handler();

    HAL_Delay (2); // request to fast, add a small delay
  }
}

/**
 * @brief  Send a CAN-message on CAN2-bus
 *         Is the CANID smaller than 0x800 the standard ID frame format is used, otherwise the extended ID frame format is used.
 * @param  CANID: specifies the CANID with which the message is to be sent
 *         DLC: specifies the number of bytes that will be sent
 *         Byte0: Data in Byte 0
 *         Byte1: Data in Byte 1
 *         Byte2: Data in Byte 2
 *         Byte3: Data in Byte 3
 *         Byte4: Data in Byte 4
 *         Byte5: Data in Byte 5
 *         Byte6: Data in Byte 6
 *         Byte7: Data in Byte 7
 * @retval none
 */
void SendCAN2_ClassicMessage ( uint32_t CANID, uint32_t DLC, uint8_t Byte0, uint8_t Byte1, uint8_t Byte2, uint8_t Byte3, uint8_t Byte4, uint8_t Byte5,
    uint8_t Byte6, uint8_t Byte7 )
{
  /* Prepare Tx Header */
  FDCAN_TxHeader.FDFormat = FDCAN_CLASSIC_CAN;
  FDCAN_TxHeader.TxFrameType = FDCAN_DATA_FRAME;
  FDCAN_TxHeader.ErrorStateIndicator = FDCAN_ESI_ACTIVE;
  FDCAN_TxHeader.BitRateSwitch = FDCAN_BRS_OFF;
  FDCAN_TxHeader.TxEventFifoControl = FDCAN_NO_TX_EVENTS;
  FDCAN_TxHeader.MessageMarker = 0;
  FDCAN_TxHeader.Identifier = CANID;
  FDCAN_TxHeader.IdType = FDCAN_STANDARD_ID;
  if ( CANID >= 0x800 )
    FDCAN_TxHeader.IdType = FDCAN_EXTENDED_ID;
  FDCAN_TxHeader.DataLength = DLC;

  /* Fill Data */
  FDCAN_TxData[0] = Byte0;
  FDCAN_TxData[1] = Byte1;
  FDCAN_TxData[2] = Byte2;
  FDCAN_TxData[3] = Byte3;
  FDCAN_TxData[4] = Byte4;
  FDCAN_TxData[5] = Byte5;
  FDCAN_TxData[6] = Byte6;
  FDCAN_TxData[7] = Byte7;

  /* Start the Transmission process */
  if ( HAL_FDCAN_AddMessageToTxFifoQ(&hfdcan2, &FDCAN_TxHeader, FDCAN_TxData) != HAL_OK )
  {
    /* Transmission request Error */
    // Error_Handler();

    HAL_Delay (2); // request to fast, add a small delay
  }
}

/**
 * @brief  Send a CANFD-message on CAN-bus, DLC(=12)
 *         Is the CANID smaller than 0x800 the standard ID frame format is used, otherwise the extended ID frame format is used.
 * @param  CANID: specifies the CANID with which the message is to be sent
 *         Byte0: Data in Byte 0
 *         Byte1: Data in Byte 1
 *         Byte2: Data in Byte 2
 *         Byte3: Data in Byte 3
 *         Byte4: Data in Byte 4
 *         Byte5: Data in Byte 5
 *         Byte6: Data in Byte 6
 *         Byte7: Data in Byte 7
 *         Byte8: Data in Byte 8
 *         Byte9: Data in Byte 9
 *         Byte10: Data in Byte 10
 *         Byte11: Data in Byte 11
 *         Byte12: Data in Byte 12
 * @retval none
 */
void SendCAN1_FDMessage_DLC12 ( uint32_t CANID, uint8_t Byte0, uint8_t Byte1, uint8_t Byte2, uint8_t Byte3, uint8_t Byte4, uint8_t Byte5, uint8_t Byte6,
    uint8_t Byte7, uint8_t Byte8, uint8_t Byte9, uint8_t Byte10, uint8_t Byte11 )
{
  /* Prepare Tx Header */
  FDCAN_TxHeader.FDFormat = FDCAN_FD_CAN;
  FDCAN_TxHeader.TxFrameType = FDCAN_DATA_FRAME;
  FDCAN_TxHeader.ErrorStateIndicator = FDCAN_ESI_ACTIVE;
  FDCAN_TxHeader.BitRateSwitch = FDCAN_BRS_OFF;
  FDCAN_TxHeader.TxEventFifoControl = FDCAN_NO_TX_EVENTS;
  FDCAN_TxHeader.MessageMarker = 0;
  FDCAN_TxHeader.Identifier = CANID;
  FDCAN_TxHeader.IdType = FDCAN_STANDARD_ID;
  if ( CANID >= 0x800 )
    FDCAN_TxHeader.IdType = FDCAN_EXTENDED_ID;
  FDCAN_TxHeader.DataLength = FDCAN_DLC_BYTES_12;

  /* Fill Data */
  FDCAN_TxData[0] = Byte0;
  FDCAN_TxData[1] = Byte1;
  FDCAN_TxData[2] = Byte2;
  FDCAN_TxData[3] = Byte3;
  FDCAN_TxData[4] = Byte4;
  FDCAN_TxData[5] = Byte5;
  FDCAN_TxData[6] = Byte6;
  FDCAN_TxData[7] = Byte7;
  FDCAN_TxData[8] = Byte8;
  FDCAN_TxData[9] = Byte9;
  FDCAN_TxData[10] = Byte10;
  FDCAN_TxData[11] = Byte11;

  /* Start the Transmission process */
  if ( HAL_FDCAN_AddMessageToTxFifoQ(&hfdcan1, &FDCAN_TxHeader, FDCAN_TxData) != HAL_OK )
  {
    /* Transmission request Error */
    // Error_Handler();

    HAL_Delay (2); // request to fast add a small delay
  }
}

/**
 * @brief  Send a CANFD-message on CAN-bus, DLC(=12)
 *         Is the CANID smaller than 0x800 the standard ID frame format is used, otherwise the extended ID frame format is used.
 * @param  CANID: specifies the CANID with which the message is to be sent
 *         Byte0: Data in Byte 0
 *         Byte1: Data in Byte 1
 *         Byte2: Data in Byte 2
 *         Byte3: Data in Byte 3
 *         Byte4: Data in Byte 4
 *         Byte5: Data in Byte 5
 *         Byte6: Data in Byte 6
 *         Byte7: Data in Byte 7
 *         Byte8: Data in Byte 8
 *         Byte9: Data in Byte 9
 *         Byte10: Data in Byte 10
 *         Byte11: Data in Byte 11
 *         Byte12: Data in Byte 12
 * @retval none
 */
void SendCAN2_FDMessage_DLC12 ( uint32_t CANID, uint8_t Byte0, uint8_t Byte1, uint8_t Byte2, uint8_t Byte3, uint8_t Byte4, uint8_t Byte5, uint8_t Byte6,
    uint8_t Byte7, uint8_t Byte8, uint8_t Byte9, uint8_t Byte10, uint8_t Byte11 )
{
  /* Prepare Tx Header */
  FDCAN_TxHeader.FDFormat = FDCAN_FD_CAN;
  FDCAN_TxHeader.TxFrameType = FDCAN_DATA_FRAME;
  FDCAN_TxHeader.ErrorStateIndicator = FDCAN_ESI_ACTIVE;
  FDCAN_TxHeader.BitRateSwitch = FDCAN_BRS_OFF;
  FDCAN_TxHeader.TxEventFifoControl = FDCAN_NO_TX_EVENTS;
  FDCAN_TxHeader.MessageMarker = 0;
  FDCAN_TxHeader.Identifier = CANID;
  FDCAN_TxHeader.IdType = FDCAN_STANDARD_ID;
  if ( CANID >= 0x800 )
    FDCAN_TxHeader.IdType = FDCAN_EXTENDED_ID;
  FDCAN_TxHeader.DataLength = FDCAN_DLC_BYTES_12;

  /* Fill Data */
  FDCAN_TxData[0] = Byte0;
  FDCAN_TxData[1] = Byte1;
  FDCAN_TxData[2] = Byte2;
  FDCAN_TxData[3] = Byte3;
  FDCAN_TxData[4] = Byte4;
  FDCAN_TxData[5] = Byte5;
  FDCAN_TxData[6] = Byte6;
  FDCAN_TxData[7] = Byte7;
  FDCAN_TxData[8] = Byte8;
  FDCAN_TxData[9] = Byte9;
  FDCAN_TxData[10] = Byte10;
  FDCAN_TxData[11] = Byte11;

  /* Start the Transmission process */
  if ( HAL_FDCAN_AddMessageToTxFifoQ(&hfdcan2, &FDCAN_TxHeader, FDCAN_TxData) != HAL_OK )
  {
    /* Transmission request Error */
    // Error_Handler();

    HAL_Delay (2); // request to fast add a small delay
  }
}

/**
 * @brief  Set duty cycle for LED
 * @param  DutyCycle_Percent: [0..100]
 * @retval none
 */
void WriteLED_Red ( GPIO_PinState State )
{
  HAL_GPIO_WritePin(LED_RT_GPIO_Port, LED_RT_Pin, State);
}

/**
 * @brief  Set duty cycle for LED
 * @param  DutyCycle_Percent: [0..100]
 * @retval none
 */
void WriteLED_Green ( GPIO_PinState State )
{
  HAL_GPIO_WritePin(LED_GN_GPIO_Port, LED_GN_Pin, State);
}

/**
  * @brief  Read from EEPROM
  * @param  BaseAddress: EEPROM start address
  * @retval Byte
  */
uint8_t EEPROM_Read_Byte ( uint16_t BaseAddress )
{
  uint8_t DataFromEEPROM;

  HAL_I2C_Mem_Read(&hi2c2, EEPROM_ADDRESS, BaseAddress, I2C_MEMADD_SIZE_16BIT, &DataFromEEPROM , 1, EEPROM_TIMEOUT);
  return DataFromEEPROM;
}

/**
  * @brief  Write to EEPROM
  * @param  BaseAddress: EEPROM start address
  *         Data: data source for write
  * @retval none
  */
void EEPROM_Write_Byte ( uint16_t BaseAddress, uint8_t Data )
{
  HAL_I2C_Mem_Write(&hi2c2, EEPROM_ADDRESS, BaseAddress, I2C_MEMADD_SIZE_16BIT, &Data, 1, EEPROM_TIMEOUT);
  HAL_Delay(EEPROM_WRITE_TIME);
}

/**
  * @brief  Read value from rx-register
  * @param  none
  * @retval Byte
  */
uint8_t RS232_Get_Byte ( void )
{
  return huart3.Instance->RDR;
}

/**
  * @brief  Wait a specific time for recieve a byte from RS232
  *         After recieve a byte the function will return.
  * @param  Timeout: specify the timeout for the function in ms
  * @retval Byte
  */
uint8_t RS232_Recieve_Byte ( uint32_t Timeout )
{
  uint8_t DataFrom232 = 0;

  HAL_UART_Receive(&huart3, &DataFrom232, 1, Timeout);
  return DataFrom232;
}

/**
  * @brief  Read value from rx-register
  * @param  Data: data source for write
  * @retval Byte
  */
void RS232_Send_Byte ( uint8_t Data )
{
  HAL_UART_Transmit(&huart3, &Data, 1, 100);
}


/**
  * @brief  Read value from rx-register
  * @param  none
  * @retval Byte
  */
uint8_t RS485_Get_Byte ( void )
{
  HAL_Delay(1);
  return huart2.Instance->RDR;
}

/**
  * @brief  Wait a specific time for recieve a byte from RS232
  *         After recieve a byte the function wil lreturn.
  * @param  Timeout: specify the timeout for the function in ms
  * @retval Byte
  */
uint8_t RS485_Recieve_Byte ( uint32_t Timeout )
{
  uint8_t DataFrom485 = 0;

  HAL_GPIO_WritePin(RS485_DE_GPIO_Port, RS485_DE_Pin, GPIO_PIN_RESET);
  HAL_UART_Receive(&huart2, &DataFrom485, 1, Timeout);
  HAL_GPIO_WritePin(RS485_DE_GPIO_Port, RS485_DE_Pin, GPIO_PIN_SET);
  return DataFrom485;
}

/**
  * @brief  Read value from rx-register
  * @param  Data: data source for write
  * @retval Byte
  */
void RS485_Send_Byte ( uint8_t Data )
{
  HAL_GPIO_WritePin(RS485_DE_GPIO_Port, RS485_DE_Pin, GPIO_PIN_RESET);
  HAL_UART_Transmit(&huart2, &Data, 1, 100);
  HAL_GPIO_WritePin(RS485_DE_GPIO_Port, RS485_DE_Pin, GPIO_PIN_SET);
}



