#include "main_hal.h"

FDCAN_TxHeaderTypeDef CAN_TxHeader; // CAN header, used for transmission
uint8_t CAN_TxData[64]; // CAN data to send
FDCAN_RxHeaderTypeDef CAN_RxHeader; // CAN header, used for reception
uint8_t CAN_RxData[64]; // CAN data, after the reception

/**
 * @brief  start additional Init
 * @param  none
 * @retval none
 */
void Additional_Init ( void )
{
  Additional_TIM6_Init(); // SW Tim
  Additional_CAN_Init();
}

/**
 * @brief  start additional Init Softwaretimer
 * @param  none
 * @retval none
 */
void Additional_TIM6_Init ( void )
{
  HAL_TIM_Base_Start_IT(&htim6);
}

/**
 * @brief  start additional Init for CAN_CANFD
 * @param  none
 * @retval none
 */
void Additional_CAN_Init ( void )
{
  // Change CAN to Silent mode
  HAL_GPIO_WritePin(CAN1_ON_GPIO_Port, CAN1_ON_Pin, GPIO_PIN_SET);
  HAL_GPIO_WritePin(CAN2_ON_GPIO_Port, CAN2_ON_Pin, GPIO_PIN_SET);

     /* Start the FDCAN module */
  if ( HAL_FDCAN_Start(&hfdcan1) != HAL_OK )
  {
    Error_Handler();
  }

  if ( HAL_FDCAN_ActivateNotification(&hfdcan1, FDCAN_IT_RX_FIFO0_NEW_MESSAGE, 0) != HAL_OK )
  {
    Error_Handler();
  }

 /* Start the FDCAN module */
 if (HAL_FDCAN_Start(&hfdcan2) != HAL_OK)
 {
   Error_Handler();
 }

 if (HAL_FDCAN_ActivateNotification(&hfdcan2, FDCAN_IT_RX_FIFO0_NEW_MESSAGE, 0) != HAL_OK)
 {
   Error_Handler();
 }

  // Change CAN from Silent to Active mode
  HAL_GPIO_WritePin(CAN1_ON_GPIO_Port, CAN1_ON_Pin, GPIO_PIN_RESET);
  HAL_GPIO_WritePin(CAN2_ON_GPIO_Port, CAN2_ON_Pin, GPIO_PIN_RESET);
}

/**
 * @brief  Rx FIFO 0 callback.
 * @param  hfdcan: pointer to an FDCAN_HandleTypeDef structure that contains
 *         the configuration information for the specified FDCAN.
 * @param  RxFifo0ITs: indicates which Rx FIFO 0 interrupts are signalled.
 *         This parameter can be any combination of @arg FDCAN_Rx_Fifo0_Interrupts.
 * @retval None
 */
void HAL_FDCAN_RxFifo0Callback ( FDCAN_HandleTypeDef *hfdcan, uint32_t RxFifo0ITs )
{
  if ( (RxFifo0ITs & FDCAN_IT_RX_FIFO0_NEW_MESSAGE) != RESET )
  {
    if ( hfdcan->Instance == FDCAN1 )
    {
      /* Retrieve Rx messages from RX FIFO0 */
      if ( HAL_FDCAN_GetRxMessage(hfdcan, FDCAN_RX_FIFO0, &CAN_RxHeader, CAN_RxData) == HAL_OK )
      {
        switch ( CAN_RxHeader.Identifier )
        {
          case 0x100:
            // Add code here when CAN-ID 0x100 was recieved
            break;
        }
      }
      else
      {
        Error_Handler();
      }
    }
    
    if ( hfdcan->Instance == FDCAN2 )
    {
      /* Retrieve Rx messages from RX FIFO0 */
      if ( HAL_FDCAN_GetRxMessage(hfdcan, FDCAN_RX_FIFO0, &CAN_RxHeader, CAN_RxData) == HAL_OK )
      {
        switch ( CAN_RxHeader.Identifier )
        {
          case 0x102:
            // Add code here when CAN-ID 0x100 was recieved
            break;
        }
      }
      else
      {
        Error_Handler();
      }
    }
  }
}

/**
 * @brief  Send a CAN-message on CAN1-bus
 *         Is the CANID smaller than 0x800 the standard ID frame format is used, otherwise the extended ID frame format is used.
 * @param  CANID: specifies the CANID with which the message is to be sent
 *         DLC: specifies the number of bytes that will be sent
 *         Byte0: Data in Byte 0
 *         Byte1: Data in Byte 1
 *         Byte2: Data in Byte 2
 *         Byte3: Data in Byte 3
 *         Byte4: Data in Byte 4
 *         Byte5: Data in Byte 5
 *         Byte6: Data in Byte 6
 *         Byte7: Data in Byte 7
 * @retval none
 */
void SendCAN1_ClassicMessage ( uint32_t CANID, uint32_t DLC, uint8_t Byte0, uint8_t Byte1, uint8_t Byte2, uint8_t Byte3, uint8_t Byte4, uint8_t Byte5,
    uint8_t Byte6, uint8_t Byte7 )
{
  /* Prepare Tx Header */
  CAN_TxHeader.FDFormat = FDCAN_CLASSIC_CAN;
  CAN_TxHeader.TxFrameType = FDCAN_DATA_FRAME;
  CAN_TxHeader.ErrorStateIndicator = FDCAN_ESI_ACTIVE;
  CAN_TxHeader.BitRateSwitch = FDCAN_BRS_OFF;
  CAN_TxHeader.TxEventFifoControl = FDCAN_NO_TX_EVENTS;
  CAN_TxHeader.MessageMarker = 0;
  CAN_TxHeader.Identifier = CANID;
  CAN_TxHeader.IdType = FDCAN_STANDARD_ID;
  if ( CANID >= 0x800 )
    CAN_TxHeader.IdType = FDCAN_EXTENDED_ID;
  CAN_TxHeader.DataLength = DLC;

  /* Fill Data */
  CAN_TxData[0] = Byte0;
  CAN_TxData[1] = Byte1;
  CAN_TxData[2] = Byte2;
  CAN_TxData[3] = Byte3;
  CAN_TxData[4] = Byte4;
  CAN_TxData[5] = Byte5;
  CAN_TxData[6] = Byte6;
  CAN_TxData[7] = Byte7;

  /* Start the Transmission process */
  if ( HAL_FDCAN_AddMessageToTxFifoQ(&hfdcan1, &CAN_TxHeader, CAN_TxData) != HAL_OK )
  {
    /* Transmission request Error */
    // Error_Handler();

    HAL_Delay (2); // request to fast, add a small delay
  }
}

/**
 * @brief  Send a CAN-message on CAN2-bus
 *         Is the CANID smaller than 0x800 the standard ID frame format is used, otherwise the extended ID frame format is used.
 * @param  CANID: specifies the CANID with which the message is to be sent
 *         DLC: specifies the number of bytes that will be sent
 *         Byte0: Data in Byte 0
 *         Byte1: Data in Byte 1
 *         Byte2: Data in Byte 2
 *         Byte3: Data in Byte 3
 *         Byte4: Data in Byte 4
 *         Byte5: Data in Byte 5
 *         Byte6: Data in Byte 6
 *         Byte7: Data in Byte 7
 * @retval none
 */
void SendCAN2_ClassicMessage ( uint32_t CANID, uint32_t DLC, uint8_t Byte0, uint8_t Byte1, uint8_t Byte2, uint8_t Byte3, uint8_t Byte4, uint8_t Byte5,
    uint8_t Byte6, uint8_t Byte7 )
{
  /* Prepare Tx Header */
  CAN_TxHeader.FDFormat = FDCAN_CLASSIC_CAN;
  CAN_TxHeader.TxFrameType = FDCAN_DATA_FRAME;
  CAN_TxHeader.ErrorStateIndicator = FDCAN_ESI_ACTIVE;
  CAN_TxHeader.BitRateSwitch = FDCAN_BRS_OFF;
  CAN_TxHeader.TxEventFifoControl = FDCAN_NO_TX_EVENTS;
  CAN_TxHeader.MessageMarker = 0;
  CAN_TxHeader.Identifier = CANID;
  CAN_TxHeader.IdType = FDCAN_STANDARD_ID;
  if ( CANID >= 0x800 )
    CAN_TxHeader.IdType = FDCAN_EXTENDED_ID;
  CAN_TxHeader.DataLength = DLC;

  /* Fill Data */
  CAN_TxData[0] = Byte0;
  CAN_TxData[1] = Byte1;
  CAN_TxData[2] = Byte2;
  CAN_TxData[3] = Byte3;
  CAN_TxData[4] = Byte4;
  CAN_TxData[5] = Byte5;
  CAN_TxData[6] = Byte6;
  CAN_TxData[7] = Byte7;

  /* Start the Transmission process */
  if ( HAL_FDCAN_AddMessageToTxFifoQ(&hfdcan2, &CAN_TxHeader, CAN_TxData) != HAL_OK )
  {
    /* Transmission request Error */
    // Error_Handler();

    HAL_Delay (2); // request to fast, add a small delay
  }
}

/**
 * @brief  Set duty cycle for LED
 * @param  DutyCycle_Percent: [0..100]
 * @retval none
 */
void WriteLED_Red ( GPIO_PinState State )
{
  HAL_GPIO_WritePin(LED_RT_GPIO_Port, LED_RT_Pin, State);
}

/**
 * @brief  Set duty cycle for LED
 * @param  DutyCycle_Percent: [0..100]
 * @retval none
 */
void WriteLED_Green ( GPIO_PinState State )
{
  HAL_GPIO_WritePin(LED_GN_GPIO_Port, LED_GN_Pin, State);
}

/**
  * @brief  Read from EEPROM
  * @param  BaseAddress: EEPROM start address
  * @retval Byte
  */
uint8_t EEPROM_Read_Byte ( uint16_t BaseAddress )
{
  uint8_t DataFromEEPROM;

  HAL_I2C_Mem_Read(&hi2c2, EEPROM_ADDRESS, BaseAddress, I2C_MEMADD_SIZE_16BIT, &DataFromEEPROM , 1, EEPROM_TIMEOUT);
  return DataFromEEPROM;
}

/**
  * @brief  Write to EEPROM
  * @param  BaseAddress: EEPROM start address
  *         Data: data source for write
  * @retval none
  */
void EEPROM_Write_Byte ( uint16_t BaseAddress, uint8_t Data )
{
  HAL_I2C_Mem_Write(&hi2c2, EEPROM_ADDRESS, BaseAddress, I2C_MEMADD_SIZE_16BIT, &Data, 1, EEPROM_TIMEOUT);
  HAL_Delay(EEPROM_WRITE_TIME);
}

/**
  * @brief  Read value from rx-register
  * @param  none
  * @retval Byte
  */
uint8_t RS232_Get_Byte ( void )
{
  return huart3.Instance->RDR;
}

/**
  * @brief  Wait a specific time for recieve a byte from RS232
  *         After recieve a byte the function will return.
  * @param  Timeout: specify the timeout for the function in ms
  * @retval Byte
  */
uint8_t RS232_Recieve_Byte ( uint32_t Timeout )
{
  uint8_t DataFromRS232 = 0;

  HAL_UART_Receive(&huart3, &DataFromRS232, 1, Timeout);
  return DataFromRS232;
}

/**
  * @brief  Read value from rx-register
  * @param  Data: data source for write
  * @retval Byte
  */
void RS232_Send_Byte ( uint8_t Data )
{
  HAL_UART_Transmit(&huart3, &Data, 1, 100);
}

